/*
 * e4common.h --
 *
 *	This file contains values and defines that are shared between the
 *	implementation and interface part of e4Graph.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__E4_COMMON_H__
#define	__E4_COMMON_H__

/*
 * These config parameters are lifted from metakit-1.8.5/include/mk4.h.
 */

/*
 * Borland C++ and C++ Builder
 */

#if defined (__BORLANDC__)
	/*
         * Borland 5.0 supports the bool datatype
         */
    #if __BORLANDC__ >= 0x500
        #define e4_BOOL 1
    #endif
    #define E4_NO_CAST
#endif

/*
 * Metrowerks CodeWarrior
 */

#if defined (__MWERKS__)
    #if __option(bool)
        #define e4_BOOL 1
        #undef bool
        #undef true
        #undef false
    #endif
    #define E4_NO_CAST
#endif

/*
 *  Microsoft Visual C++
 */

#if defined (_MSC_VER)
	/*
         * MSVC 5.0 supports the bool datatype, MSVC 4.x has no namespaces
         */
    #if _MSC_VER >= 1100
        #define e4_BOOL 1
    #endif
    #include <malloc.h>
    #define E4_NO_CAST
#endif

/*
 * Symantec C++, Contributed by John Fletcher.
 */

#if defined(__SC__)
#ifdef _BOOL_DEFINED
#define e4_BOOL 1
#endif
#endif

/*
 * GNU C++ has a bool.
 */

#if defined (__GNUC__)
#define e4_BOOL 1
#endif

/*
 * If no bool definition is provided, define one.
 */

#if !e4_BOOL
    #define false 0
    #define true 1
    #define bool int
#endif

/*
 * Add casts for compilers that need them.
 */

#ifdef	E4_NO_CAST
#define	E4_CONST_VOID_STAR
#define E4_CONST_CHAR_STAR
#else
#define	E4_CONST_VOID_STAR	(const void *)
#define E4_CONST_CHAR_STAR	(const char *)
#endif

/*
 * Public #define values.
 */

#define E4_MARKERNOTFOUND	-1	/* A marker with the given name
					 * could not be found in this
					 * storage. */

#define E4_VERTEXNOTFOUND	-1	/* A vertex with the given name
					 * could not be found in this
					 * node. */

#define E4_VERTEXNOTCREATED	-1	/* Some kind of failure during
					 * vertex creation. */

#define E4_NODENOTFOUND		-2	/* A requested node could not
					 * be found. */

#define E4_ROOTINDEX		0	/* ID of root node in each marker. */

#define E4_RANKNOTUSED		-1	/* Constant to pass to operations
					 * that take a rank, to denote that
					 * the value is meaningless. */

#define E4_INVALIDUNIQUEID	-1	/* Constant that denotes an invalid
					 * unique ID. */

/*
 * Get needed definitions from the operating system:
 */

#include <string.h>

/*
 * Get needed definitions from the standard C++ library.
 */

#include <stdlib.h>

/*
 * Include forward declarations of all classes in e4Graph.
 */

#include "e4dll.h"

/*
 * Include the dynamic string facility.
 */

#include "e4dstring.h"

/*
 * Include the integer stack facility.
 */

#include "e4intstack.h"

/*
 * Include declarations for all the known storage drivers.
 */

#include "e4drivers.h"

/*
 * Include the declarations for the e4_HashTable facility.
 */

#include "e4hash.h"

/*
 * Enumerate the various kinds of reference counted entities:
 *
 * E4_RKINVALID		Illegal value.
 * E4_RKSTORAGE		This instance is a storage.
 * E4_RKMARKER		This instance is a marker.
 * E4_RKNODE		This instance is a node.
 * E4_RKVERTEX		This instance is a vertex.
 */

typedef enum e4_RefKind {
    E4_RKINVALID =		-1,
    E4_RKSTORAGE =		0,
    E4_RKMARKER =		1,
    E4_RKNODE =			2,
    E4_RKVERTEX =		3
} e4_RefKind;

/*
 * Enumerate the various vertex types:
 *
 * E4_VTUNKNOWN		Illegal value.
 * E4_VTNODE		This vertex contains a node.
 * E4_VTINT		This vertex contains a 32-bit integer.
 * E4_VTDOUBLE		This vertex contains a 64-bit floating point number.
 * E4_VTSTRING		This vertex contains a NULL-terminated string.
 * E4_VTBINARY		This vertex contains a binary unstructured value.
 */

typedef enum e4_VertexType {
    E4_VTUNKNOWN =		-1,
    E4_VTNODE =			0,
    E4_VTINT =			1,
    E4_VTDOUBLE =		2,
    E4_VTSTRING =		3,
    E4_VTBINARY =		4,
    E4_VTLASTVERTEXTYPE =	5
} e4_VertexType;

/*
 * The following enumeration describes how vertices are moved from
 * another node into this node. The argument determines how the
 * vertex is inserted into this node:
 *
 * E4_IONONE		Illegal value.
 * E4_IOAT		Insert at numeric position (zero based).
 * E4_IOFIRST		Insert as zeroth vertex.
 * E4_IOLAST		Insert as last vertex.
 * E4_IOBEFORE		Insert before another named vertex.
 * E4_IOAFTER		Insert after another named vertex.
 */

typedef enum e4_InsertOrder {
    E4_IONONE =			0,
    E4_IOAT =			1,
    E4_IOFIRST =		2,
    E4_IOLAST =			3,
    E4_IOBEFORE =		4,
    E4_IOAFTER =		5,
    E4_IOLASTINSERTORDER =	6
} e4_InsertOrder;

/*
 * The following enumeration describes the different methods of
 * operation for an e4_Visitor instance:
 *
 * E4_VMUNKNOWN		Illegal value.
 * E4_VMSTORAGE		Visit every vertex in a storage in some random order.
 * E4_VMNODE		Visit every vertex in a node in rank order.
 * E4_VMNODERANDOM	Visit every vertex in a node in some random order.
 */

typedef enum e4_VisitMethod {
    E4_VMUNKNOWN =		0,
    E4_VMSTORAGE =		1,
    E4_VMNODE =			2,
    E4_VMNODERANDOM =		3,
    E4_VMLASTMETHOD =		4
} e4_VisitMethod;

/*
 * The following bit fields define different filters to apply to exclude
 * vertices from being visited. These bit fields can be ORed together to
 * yield more complex filters.
 */

#define E4_VFNONE	0		/* Always visit.		*/
#define	E4_VFNAME	(1 << 0)	/* Visit only if name equals.	*/
#define E4_VFTYPE	(1 << 1)	/* Visit only if type equals.	*/

/*
 * This structure is used to pass a value to/from an untyped vertex
 * access function.
 */

typedef struct e4_Binary {
    int nbytes;			/* How big is the value? */
    void *bytes;		/* The value itself. */
} e4_Binary;

/*
 * A way to transmit arbitrary values:
 */

typedef struct e4_ValueImpl {
    e4_VertexType vertexType;	/* Type of the value being passed. */
    union {
        e4_NodeImpl *n;
	int i;
        double d;
        char *s;
        e4_Binary b;
    } u;
} e4_ValueImpl;

/*
 * The following enum describes the various elements of a storage, used
 * to retrieve statistics for each element usage.
 */

typedef enum e4_Space {
    E4_SPMARKER = 0,
    E4_SPNODE = 1,
    E4_SPVERTEX = 2,
    E4_SPNAME = 3,
    E4_SPSTRING = 4,
    E4_SPINT = 5,
    E4_SPDOUBLE = 6,
    E4_SPBINARY = 7,
    E4_SPLAST = 8
} e4_Space;

/*
 * The following enum describes the different statistics collected for
 * each space. Some storage drivers may not collect all these statistics.
 */

typedef enum e4_SpaceStat {
    E4_SSUSED = 0,
    E4_SSAVAIL = 1,
    E4_SSFREED = 2,
    E4_SSALLOC = 3,
    E4_SSLAST = 4
} e4_SpaceStat;

/*
 * Include the meta cache information header file:
 */

#include "e4metacache.h"

/*
 * Callback function typedefs. These types are used to declare functions that
 * will be called when significant events happen in e4Graph.
 */

typedef void (*e4_MarkerAddCallbackFunction)(void *clientData, e4_Marker m);
typedef void (*e4_MarkerDelCallbackFunction)(void *clientData, e4_Marker m);
typedef void (*e4_NodeAddCallbackFunction)(void *clientData, e4_Node n);
typedef void (*e4_NodeDelCallbackFunction)(void *clientData, e4_Node n);
typedef void (*e4_VertexAddCallbackFunction)(void *clientData, e4_Vertex v);
typedef void (*e4_VertexDelCallbackFunction)(void *clientData, e4_Vertex v);
typedef void (*e4_VertexModCallbackFunction)(void *clientData, e4_Vertex v);

/*
 * This enumeration defines the kinds of events for which callbacks will
 * be called.
 */

typedef enum e4_CallbackKind {
    E4_CBADDMARKER = 0,		/* A marker was added to the storage. */
    E4_CBDELMARKER = 1,		/* A marker is about to be deleted. */
    E4_CBADDNODE = 2,		/* A node was added to the storage. */
    E4_CBDELNODE = 3,		/* A node is about to be deleted. */
    E4_CBADDVERTEX = 4,		/* A vertex was added to the storage. */
    E4_CBDELVERTEX = 5,		/* A vertex is about to be deleted. */
    E4_CBMODVERTEX = 6		/* A vertex value was modified. */
} e4_CallbackKind;

#endif	/* !__E4_COMMON_H__ */
