/*
 * e4dstring.h --
 *
 * Dynamic string facility for e4Graph, modelled after the Tcl 8.0.4
 * Tcl_DString facility.
 *
 * Copyright (c) 1991-1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__E4_DSTRING_H__
#define	__E4_DSTRING_H__

/*
 * How big is the static space?
 */

#define	E4_DSTRING_STATIC_SIZE		200

/*
 * The DString class:
 */

class e4_DLL e4_DString {
private:

    /*
     * The string itself.
     */

    char *buf;

    /*
     * The static space.
     */

    char staticSpace[E4_DSTRING_STATIC_SIZE + 1];

    /*
     * How much space is available?
     */

    int spaceAvailable;

    /*
     * And how long is the current value of the string?
     */

    int length;

public:

    /*
     * Constructor:
     */

    e4_DString();

    /*
     * Destructor:
     */

    ~e4_DString();

    /*
     * Append a string to this DString.
     */

    void Append(const char *str, int slen);

    /*
     * Get (a copy of) the current value of the string.
     */

    char *Get();

    /*
     * Get the length of the string.
     */

    int Length();

    /*
     * Set the length of the string.
     */

    int SetLength(int newlength);
};

#endif	/* __E4_DSTRING_H__ */
